# Prerequisites:
* How the Prerequisites docs are organized:
  * This README.md is meant to be a high level overview of prerequsites.
  * /docs/guides/prerequsites/(some_topic).md files are meant to offer more specific guidance on prerequisites while staying generic. 
  * The /docs/guides/deployment_scenarios/(some_topic).md files may also offer additional details on prerequesites specific to the scenario. 
* Prerequisites vary depending on deployment scenario, thus a table is used to give an overview.
* Note for future edits: The following table was generated using tablesgenerator.com/markdown_tables, recommended to copy the table's raw text contents, visit tablesgenerator.com/markdown_tables, and File -> Paste table data when edits are needed.

| Prerequisites(rows) vs Deployment Scenarios(columns)                                                                                                                                                                                           | QuickStart                                                                                                                                                                                                                                                                                                | Internet Connected                                                                                                                                                                                                      | Internet Disconnected                                                                                                                                                                            |
|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| **[Minimum Hardware Requirements](minimum_hardware_requirements.md)** (vCPU, Memory, and Disk Storage for Big Bang in a production system)                                                                                                     | The quickstart's demo values are known to work against a single VM with 8 CPU cores, 32 GB ram, and a 100 GB root partition. (It should be possible to lower the helm values to fit on 4 CPU cores and 16 GB ram, and less disk space.)                                                                                                                                                               | Appropriately sized CSP instances (The hardware requirements vary greatly depending on the enabled packages, internal database use, replicas, nodes, and memory/cpu requests.)                                                                                                                                                                        | Appropriately sized virtual or bare metal machines (The hardware requirements vary greatly depending on the enabled packages, internal database use, replicas, nodes, and memory/cpu requests.)                                                                                                                                |
| **[OS Preconfigured](os_preconfiguration.md) and Prehardened**  (OS and level of hardening required depends on AO)                                                                                                                             | Prerequisite  Recommended: A non-hardened single VM, (the quickstart has copy pasteable commands to satisfy the configuration requirements.)                                                                                                                                                           | Prerequisite  (CSPs usually have marketplaces with pre-hardened VM images)                                                                                                                                              | Prerequisite  (configured to AO's risk tolerance / mission needs)                                                                                                                                |
| **[Kubernetes Distribution Preconfigured](kubernetes_preconfiguration.md) to Best Practices and Prehardened**  (Any CNCF Kubernetes Distribution will work as long as an AO signs off on it)                                                   | k3d is recommended for demos (It can be quickly installed and setup using copy pasteable commands in the quickstart, ships with a dockerized LB, works on every cloud, and bare metal)                                                                                                                                                                                     | Prerequisite  (https://repo1.dso.mil/platform-one/distros)                                                                                                                                                              | Prerequisite  (users are responsible for airgap image import of container images needed by chosen Kubernetes Distribution)                                                                       |
| **Default Storage Class**  ((for Dynamic PVCs), the SC needs to support RWX (Read Write Many) Access Mode to support HA deployment of all BigBang AddOns)                                                                                      | Presatisfied*  (*if using k3d, which has dynamic local volume storage class baked in)                                                                                                                                                                                                                     | Prerequisite  It's recommended that users start with a CSP specific or Kubernetes Distro provided storage class                                                                                                         | Prerequisite  [(These docs compare Cloud Agnostic Storage Solutions)](../../k8s-storage/README.md#kubernetes-storage-options)                                                                    |
| **Support for Automated Provisioning of Service Type Load Balancer**  (is recommended)                                                                                                                                                         | Presatisfied*  (*if using k3d, which ships with the ability to add flags to treat the VM's port 443 as Kubernetes Service of Type LB's port 443, automation in the quick start repo leverages these flags)                                                                                                | Prerequisite  Kubernetes Distributions usually have CSPs specific flags you can pass to the kube-apiserver to support auto provisioning of CSP LBs.                                                                     | Prerequisite  [(See docs for guidance on bare metal and no IAM scenarios)](kubernetes_preconfiguration.md#service-of-type-load-balancer)                                                         |
| **Access to Container Images**  (IronBank Image Pull Credentials or AirGap import from .tar.gz's)                                                                                                                                              | Prerequisite  (Anyone can go to login.dso.mil, and self register against P1's SSO. That can be used to login to registry1.dso.mil to generate image pull credentials for the QuickStart)                                                                                                                  | BigBang customers are recommended to use ask their BB Customer Liaison's for an IronBank Image pull robot account, which lasts 6 months.                                                                                | Prerequisite  (Airgap import of container images, [BigBang Releases](https://repo1.dso.mil/platform-one/big-bang/bigbang/-/releases) includes a .tar.gz of IronBank Images)                      |
| **Customer Controlled Private Git Repo**  (for GitOps, the Cluster needs network access & Credentials for Read Access to said Git Repo)                                                                                                        | Substituted  (the turn key demo uses a non GitOps workflow )                                                                                                                                                                                                           | Prerequisite  (or follow Air gap docs)                                                                                                                                                                                  | Prerequisite  (Air gap docs assist with provisioning an ssh based git repo)                                                                                                                      |
| **Encrypt Secrets as Code**  (Use SOPS + CSP KMS or PGP to encrypt secrets that need to be stored in the GitRepo)                                                                                                                              | Substituted  (Quickstart demo leverages clear text secrets in local files / outside of git)                                                                                                                                                                                                                | Prerequisite  (CSP KMS and IAM is more secure that gpg key pair)                                                                                                                                                        | Prerequisite  (Use CSP KMS if available, PGP works universally, [Flux requires the private PGP key to not have a passphrase](https://toolkit.fluxcd.io/guides/mozilla-sops/#generate-a-gpg-key)) |
| **Install and Configure Flux**  (Flux needs Git Repo Credentials & CSP IAM rights for KMS decryption or a kubernetes secret containing a private PGP decryption key)                                                                           | Presatisfied  (Quickstart demo only needs installation not configuration)                                                                                                                                                                      | Prerequisite  (see BigBang docs, [flux docs](https://toolkit.fluxcd.io/components/source/gitrepositories/#spec-examples) are also a good resource for this)                                                             | Prerequisite  (see BigBang docs)                                                                                                                                                                 |
| **HTTPS Certificates**                                                                                                                                                                                                                         | Presatisfied  (The quickstart deploys *.bigbang.dev HTTPS Certificate, from the bigbang repo, a non-expired cleartext copy of the cert is kept there for dev/demo purposes, alternatively mkcert can be used to generate demo certs for arbitrary DNS names that will only be trusted by the laptop that provisioned the mkcert cert) | Prerequisite  (HTTPS cert is provided by consumer)                                                                                                                                                                      | Prerequisite  (HTTPS cert is provided by consumer)                                                                                                                                               |
| **DNS**                                                                                                                                                                                                                                        | Edit your Laptop's host file (/etc/hosts, C:\Windows\System32\drivers\etc\hosts), or use something like AWS VPC Private DNS and [sshuttle](https://github.com/sshuttle/sshuttle) to point to host VM (if using k3d)                                                                                       | Prerequisite  (point DNS names to Layer 4 CSP LB)                                                                                                                                                                       | Prerequisite  (point DNS names to L4 LB)                                                                                                                                                         |
| **HTTPS Certificate, DNS Name, and hostnames in BigBang's helm values must match**  (in order for Ingress to work correctly.)                                                                                                                  | QuickStart leverages `*.bigbang.dev` HTTPS cert, and the BigBang Helm Chart's values.yaml's hostname defaults to bigbang.dev, just need to ensure multiple hostfile entries like "grafana.bigbang.dev " exist, or if you have access to DNS a wildcard entry to map CNAME `*.bigbang.dev` to k3d VM's IP  | Prerequisite  (update bigbang helm values in git repo so hostnames match HTTPS cert)                                                                                                                                    | Prerequisite  (update bigbang helm values in git repo so hostnames match HTTPS cert)                                                                                                             |
| **SSO Identity Provider**  (Prerequisite for SSO Authentication Proxy feature)                                                                                                                                                                 | Presatisfied*  (*depending on which quick start config is used), There's exists a demo SSO config that leverages P1's CAC enabled SSO, it's coded to only work for localhost to balance turn key demo functionality against security concerns.                                                            | Prerequisite  (You don't have to use Keycloak, you can use any OIDC/SAML Identity Provider)  | Prerequisite*  (Install your own Keycloak cluster), leverage a pre-existing airgap SSO solution, or configure to not use SSO* if not needed for the use case)                                        |
| **Ops Team to integrate, configure, and maintain BigBang**  (needed skillsets: DevOps IaC/CaC all the things, automate most the things, document the rest, linux administration, productionalization and maintenance of a Kubernetes Cluster.) | QuickStart Demo is designed to be self service.                                                                                                                                                                                                                                                           | Prerequisite  (BigBang Customer Integration Engineers are available to help long term Ops teams.)                                                                                                                       | Prerequisite                                                                                                                                                                                     |
